/*
	Simple tool to generate random data sets
	for switched linear regression

	Usage: gendata datafile N n p [noise_std [add_bias]]
	
	Other parameters are hard coded below
	
	
	-- by F. Lauer, 2011.
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

/*
	Gaussian noise
	with the polar form of the Box-Muller transformation 
*/
double gaussian(void) {
         double x1, x2, w, y1, y2;
 
         do {
                 x1 = 2.0 * drand48() - 1.0;
                 x2 = 2.0 * drand48() - 1.0;
                 w = x1 * x1 + x2 * x2;
         } while ( w >= 1.0 );

         w = sqrt( (-2.0 * log( w ) ) / w );
         y1 = x1 * w;
         y2 = x2 * w;
         
         return y1;
}

/*
	Main function
*/
int main(int argc, char *argv[]) {
	
	const double w_max = 5;		// true parameters are within [-w_max, +w_max]
	const int gaussian_noise = 1;	// use uniform noise if 0
	
	int N, n, p,i,j,k;
	int S;
	double x, y, e, sigma_noise;
	FILE *fd, *fl, *fm;
	char filename[200];
	char labelfile[250];
	int affine;

	double sum_e = 0.0;
	

	if(argc < 5) {
		printf("Usage: gendata datafile N n p [noise_std [add_bias]] \n");
		return 1;
	}
	N = atoi(argv[2]);
	n = atoi(argv[3]);
	p = atoi(argv[4]);

	if(argc < 6)
		sigma_noise = 1.0;
	else 
		sigma_noise = atof(argv[5]);

	if(argc < 7 )
		affine = 0;
	else {
		affine = 1;
		p += 1;
	}
	
	// Output true labels to datafile.labels
	strcpy(labelfile,argv[1]);
	strcat(labelfile,".labels");
	
	// Make files
	if( ((fd = fopen(argv[1], "w")) == NULL) || ( (fl = fopen(labelfile, "w")) == NULL) || ( (fm = fopen("data.mat", "w")) == NULL)) {
		printf("Error in creating file.\n");
		return 1;
	}


	fprintf(fd, "%d %d\n",N,p);	// Header for data file
	
	
	// Draw random parameters
	double *w = (double*)malloc(sizeof(double) * n * p);
		
	srand48(getpid());
	for (j=0;j<n;j++) {
		printf("w_%d = [",j);
		for(k=0;k<p;k++) {
			w[j*p + k] = w_max * (2*drand48() - 1);	// drand48() is in [0,1)
			printf("%lf, ",w[j*p+k]);
		}		
		printf("]\n");
	}
	
	srand(getpid()); 
	// generate data
	for(i=0;i<N;i++) {
		// random assignment lambda_i
		S = (int)(rand() % n);		
		fprintf(fl,"%d\n",S);
		
		// y = w_lambdai' * x_i
		y = 0;
		for(k=0;k<p-1;k++) {
			// random x_ik
			x = 10*drand48() - 5; 	// x is within [-5,5]
			fprintf(fd,"%lf ",x);
			fprintf(fm,"%lf ",x); // Copy to matlab ascii file format (without header)
			
			// linear output: w_lamndai_k * xik
			y += w[S*p+k] * x;
		}
		// add constant for affine models
		if(affine) 
			x = 1;
		else
			x =  10*drand48() - 5; 
		
		fprintf(fd,"%lf ",x);
		fprintf(fm,"%lf ",x); // Copy to matlab format
		
		// linear output: w_lamndai_k * xik
		y += w[S*p+k] * x;
		
		
		if(gaussian_noise == 0) {	
			// add uniform centered noise to output
			e = sigma_noise * (2*drand48() - 1 );
		}
		else {
			// add gaussian noise
			e = sigma_noise * gaussian();
		}
		fprintf(fd,"%lf\n",y + e);	
		fprintf(fm,"%lf\n",y + e);	// for matlab		
		
		sum_e += e*e;
		
	}
	printf("Mean Squared Error with true parameters = %e\n",sum_e/N);
	
	fclose(fd);fclose(fl);fclose(fm);
	free(w);
	return 0;
}

